@@ -28,25 +28,26 @@ import Test.HUnit
 
 -- ---------------------------------------------------------------------
 
-data GHCVersion = GHC94
-           | GHC96
+data GHCVersion = GHC98
+           | GHC910
      deriving (Eq, Ord, Show)
 
 ghcVersion :: GHCVersion
-#if MIN_VERSION_ghc(9,6,0)
-ghcVersion = GHC96
+#if MIN_VERSION_ghc(9,10,0)
+ghcVersion = GHC910
 #else
-ghcVersion = GHC94
+ghcVersion = GHC98
 #endif
 
 -- | Directories to automatically find roundtrip tests
 testDirs :: [FilePath]
 testDirs =
   case ghcVersion of
-    GHC94  -> ["ghc710", "ghc80", "ghc82", "ghc84", "ghc86", "ghc88", "ghc810", "ghc90", "ghc92", "ghc94"]
-    GHC96  -> ["ghc710", "ghc80", "ghc82", "ghc84", "ghc86", "ghc88", "ghc810", "ghc90", "ghc92", "ghc94", "ghc96"]
-    -- GHC96  -> ["ghc96"]
-    -- GHC96  -> ["ghc96-copied"]
+    GHC98 -> ["pre-ghc910"]
+    GHC910 -> ["pre-ghc910", "ghc910"]
+    -- GHC910  -> ["ghc910"]
+    -- GHC910  -> ["ghc910-copied"]
+    -- GHC910  -> ["ghc910",  "ghc910-copied"]
 
 -- ---------------------------------------------------------------------
 
@@ -60,14 +61,10 @@ main = hSilence [stderr] $ do
      then exitFailure
      else return () -- exitSuccess
 
-transform :: IO ()
-transform = hSilence [stderr] $ do
+transform :: IO (Counts,Int)
+transform = do
   let libdir = GHC.Paths.libdir
-  cnts <- fst <$> runTestText (putTextToHandle stdout True) (transformTestsTT libdir)
-  putStrLn $ show cnts
-  if errors cnts > 0 || failures cnts > 0
-     then exitFailure
-     else return () -- exitSuccess
+  runTestText (putTextToHandle stdout True) (transformTestsTT libdir)
 
 -- ---------------------------------------------------------------------
 
@@ -141,25 +138,19 @@ mkTests = do
   roundTripTests <- findTests libdir
   roundTripBalanceCommentsTests <- findTestsBC libdir
   roundTripMakeDeltaTests <- findTestsMD libdir
-  -- prettyRoundTripTests <- findPrettyTests libdir
   return $ TestList [
                       internalTests,
                       roundTripTests
                    ,
                      (transformTests libdir)
-                   , (failingTests libdir)
+                   ,
+                      (failingTests libdir)
                    ,
                      roundTripBalanceCommentsTests
                    ,
                      roundTripMakeDeltaTests
                     ]
 
--- Tests that are no longer needed
-                    -- , noAnnotationTests
-                    -- ,
-                    --   prettyRoundTripTests
-                    -- ,
-
 failingTests :: LibDir -> Test
 failingTests libdir = testList "Failing tests"
   [
@@ -167,6 +158,8 @@ failingTests libdir = testList "Failing tests"
 
   -- We do not capture EOF location very well any more
     mkTestModBad libdir "T10970a.hs"
+  -- Injecting CPP comments adjacent to existing ones is a problem
+  , mkTestModBad libdir "CppComment.hs"
   ]
 
 
@@ -183,9 +176,9 @@ mkParserTestMD libdir dir fp = mkParsingTest (roundTripTestMD libdir) dir fp
 
 formatTT :: ([([Char], Bool)], [([Char], Bool)]) -> IO ()
 formatTT (ts, fs) = do
-  when (not . null $ tail ts) (do
+  when (not . null $ (drop 1) ts) (do
     putStrLn "Pass"
-    mapM_ (putStrLn . fst) (tail ts)
+    mapM_ (putStrLn . fst) ((drop 1) ts)
     )
   when (not . null $ fs) (do
     putStrLn "Fail"
@@ -202,60 +195,23 @@ tt' = do
   let libdir = GHC.Paths.libdir
   runTestText (putTextToHandle stdout True) $ TestList [
 
-    -- mkTestModChange libdir rmDecl7 "RmDecl7.hs"
+    -- mkParserTest libdir "ghc98" "ModuleComments1.hs"
+    -- mkParserTestBC libdir "ghc98" "MonoidsFD1.hs"
+    -- mkParserTestMD libdir "ghc98" "ModuleComments1.hs"
 
-    -- mkTestModChange libdir changeLocalDecls  "LocalDecls.hs"
-    -- mkTestModChange libdir changeLayoutLet2 "LayoutLet2.hs"
-    -- mkTestModChange libdir addLocaLDecl5  "AddLocalDecl5.hs"
-
-   -- mkTestModChange libdir rmDecl1 "RmDecl1.hs"
-
-   -- mkTestModChange libdir rmDecl4 "RmDecl4.hs"
-    -- mkParserTestMD libdir      "ghc92" "Foo.hs"
-    -- mkParserTest libdir      "ghc92" "Foo.hs"
-    -- mkParserTestMD libdir      "ghc92" "Foo.hs"
-    -- mkParserTest libdir      "ghc92" "Foo2.hs"
-    -- mkParserTest libdir      "ghc710" "EmptyMostly.hs"
-    -- mkParserTestBC libdir "ghc710" "Control.hs"
-    -- mkParserTestBC libdir "ghc92" "CommentPlacement3.hs"
-    -- mkParserTestBC libdir "ghc92" "TopLevelSemis.hs"
-
-    -- mkParserTest libdir      "ghc92" "ConstructorComment.hs"
-    -- mkParserTest libdir      "ghc92" "Binary.hs"
-    -- mkParserTest libdir      "ghc92" "Observer.hs"
-    -- mkParserTest libdir      "ghc92" "Observer1.hs"
-
-   -- mkTestModChange libdir addLocaLDecl1  "AddLocalDecl1.hs"
-   -- mkTestModChange libdir addLocaLDecl3  "AddLocalDecl3.hs"
-
-    -- mkParserTestBC libdir "ghc710" "MultiParamTypeClasses.hs"
-
-    -- mkParserTestBC libdir "ghc710" "DataFamilies.hs"
-    -- mkParserTestBC libdir "ghc710" "Cpp.hs"
-    -- mkParserTestBC libdir "ghc80" "T4139.hs"
-
-    -- mkParserTestBC libdir "ghc92" "Checkpoint.hs"
-
-    -- mkParserTestBC libdir "ghc92" "CommentPlacement6.hs"
-    -- mkParserTest libdir "ghc92" "CommentPlacement6.hs"
-
-    -- mkParserTest libdir "ghc92" "TopLevelSemis.hs"
-    -- mkParserTestBC libdir "ghc92" "TopLevelSemis.hs"
-    -- mkParserTestMD libdir "ghc92" "TopLevelSemis.hs"
-
-
-    -- mkParserTest libdir "ghc96" "T11671_run.hs"
-
-    -- mkParserTest libdir "ghc96" "LexerM.hs"
-    -- mkParserTestBC libdir "ghc96" "LexerM.hs"
-
-    -- mkParserTest libdir "ghc94" "Haddock.hs"
-    -- mkParserTest libdir "ghc94" "Haddock1.hs"
-    mkParserTestBC libdir "ghc94" "Haddock1.hs"
+    -- mkParserTest libdir "ghc910" "LinearLet.hs"
+    -- mkParserTest libdir "ghc910" "Generic.hs"
+    -- mkParserTest libdir "ghc910" "Expression.hs"
+    -- mkParserTest libdir "ghc910" "ConstructorArgs.hs"
+    -- mkParserTest libdir "ghc910" "CppComment.hs"
+    -- mkParserTest libdir "ghc910" "Class.hs"
+    -- mkParserTest libdir "ghc910" "Test138.hs"
+    mkParserTest libdir "vect" "DiophantineVect.hs"
 
    -- Needs GHC changes
 
 
+
     ]
 
 testsTT :: LibDir -> Test
