@@ -31,16 +31,17 @@ module Test.Common (
 
 
 import Language.Haskell.GHC.ExactPrint
--- import Language.Haskell.GHC.ExactPrint.ExactPrint
 import Language.Haskell.GHC.ExactPrint.Utils
 import Language.Haskell.GHC.ExactPrint.Parsers
 import Language.Haskell.GHC.ExactPrint.Preprocess
 
 import qualified Control.Monad.IO.Class as GHC
-import qualified GHC           as GHC hiding (parseModule)
--- import qualified GHC.Data.Bag          as GHC
+import GHC hiding (moduleName)
+import GHC.Driver.Errors.Types
 import qualified GHC.Driver.Session    as GHC
--- import qualified GHC.Utils.Error       as GHC
+import GHC.Types.Error
+import GHC.Utils.Error
+import GHC.Utils.Outputable (renderWithContext, defaultSDocContext, vcat)
 
 import qualified GHC.LanguageExtensions as LangExt
 
@@ -108,12 +109,11 @@ noChange _libdir parsed = return parsed
 
 changeBalanceComments :: Changer
 changeBalanceComments _libdir top = do
-  let (GHC.L l p) = makeDeltaAst top
-  -- let (GHC.L l p) = top
+  -- let (GHC.L l p) = makeDeltaAst top
+  let (GHC.L l p) = top
   let decls0 = GHC.hsmodDecls p
-      (decls,_,w) = runTransform (balanceCommentsList decls0)
+      decls = balanceCommentsList decls0
   let p2 = p { GHC.hsmodDecls = decls}
-  debugM $ "changeBalanceComments:\n" ++ unlines w
   return (GHC.L l p2)
 
 changeMakeDelta :: Changer
@@ -146,6 +146,22 @@ genTest libdir f origFile expectedFile  = do
               cppStatus = if useCpp then Just orig else Nothing
           return $ Right Report {..}
 
+showErrorMessages :: Messages GhcMessage -> String
+showErrorMessages msgs =
+  renderWithContext defaultSDocContext
+    $ vcat
+    $ pprMsgEnvelopeBagWithLocDefault
+    $ getMessages
+    $ msgs
+
+-- showErrorMessages :: Messages GhcMessage -> String
+-- showErrorMessages msgs =
+--   renderWithContext defaultSDocContext
+--     $ vcat
+--     $ pprMsgEnvelopeBagWithLocDefault
+--     $ getMessages
+--     $ msgs
+
 
 mkDebugOutput :: FilePath -> String -> String
               -> GHC.ParsedSource -> String
@@ -168,7 +184,9 @@ runRoundTrip :: LibDir
              -> [GHC.LEpaComment]
              -> IO (String, GHC.ParsedSource)
 runRoundTrip libdir f !parsedOrig cs = do
+  -- putStrLn $ "comments:" ++ showAst cs
   let !parsedOrigWithComments = insertCppComments parsedOrig cs
+  -- putStrLn $ "parsedOrigWithComments:" ++ showAst parsedOrigWithComments
   pmod <- f libdir parsedOrigWithComments
   let !printed = exactPrint pmod
   return (printed, pmod)
@@ -199,7 +217,7 @@ getModSummaryForFile fileName = do
 
   let mm = filter (\(mfn,_ms) -> mfn == Just cfileName) cgraph
   case mm of
-   [] -> return Nothing
-   fs -> return (Just (snd $ head fs))
+   (f:_) -> return (Just (snd f))
+   _ -> return Nothing
 
 -- ---------------------------------------------------------------------
